#include <algorithm>
#include <cstdio>
#include <cmath>
#include <cstdlib>
using namespace std;

typedef long long Z;
const Z inf (2e9);

unsigned dram_rand () {
	return unsigned(rand()) << 16 | unsigned(rand());
}

struct Rec {
	Z left, right, mn, mx, mindiff, size;

	Rec (Z x):
		left (x), right (x),
		mn (x), mx (x),
		mindiff (inf), size (1) {}
		
	Rec (Z l, Z r, Z n, Z x, Z d, Z z):
		left (l), right (r), mn (n), mx (x), mindiff (d), size (z) {}
};

Rec operator+ (const Rec &a, const Rec &b) {
	Rec res (
		a.left,
		b.right,
		min (a.mn, b.mn),
		max (a.mx, b.mx),
		min (	min (a.mindiff, b.mindiff),
	    		abs (a.right - b.left) ),
		a.size + b.size
	);
	return res;
}

struct T {
	Z x;
	Rec r;
	int prio;
	T *left, *right;
	
	void update () {
		r = Rec (x);
		if (left) r = left->r + r;
		if (right) r = r + right->r;
	}
	
	T (Z x_): x (x_), r(x_), prio(dram_rand()), left(0), right(0) {}
};

void dump (T *x, int ind = 0) {
	if (x) {
		dump (x->left, ind + 1);
		for (int i = 1; i <= ind; i ++) fprintf (stderr, "    ");
		fprintf (stderr, "%lld = %lld %lld %lld %lld %lld %lld\n", x->x, x->r.left, x->r.right, x->r.mn, x->r.mx, x->r.mindiff, x->r.size);
		dump (x->right, ind + 1);
	} else {
		for (int i = 1; i <= ind; i ++) fprintf (stderr, "    ");
		fprintf (stderr, "x\n");
	}
}

T* merge (T *x, T *y) {
	if (x) {
		if (y) {

			if (x->prio > y->prio) {
				x->right = merge (x->right, y);
				
				x->update ();
				return x;
			} else {
				y->left = merge (x, y->left);
				
				y->update ();
				return y;
			}

		} else return x;
	} else return y;
}

void split (T *x, int k, T **l, T **r) {
	//fprintf (stderr, "split %p %d\n", x, k);
	//dump (x);
	//fprintf (stderr, "------\n");
	if (x) {
		int lsz = x->left ? x->left->r.size : 0;
		//dump (x->left);
		//fprintf (stderr, "lsz %d\n", lsz);
		if (k <= lsz) {
			
			//fprintf (stderr, "LEFT\n");
			split (x->left, k, l, &x->left);
			x->update ();
			*r = x;
		} else {
			//fprintf (stderr, "RIGHT\n");
			split (x->right, k - lsz - 1, &x->right, r);
			x->update ();
			*l = x;
		}
	} else *l = *r = 0;
}

inline
T* alloc (T **pool, Z val) {
	return new ((*pool) ++) T(val);
}

int main () {
	T *root= 0, *pool = (T*) malloc (sizeof(T) * 300000);
	freopen ("atom.in", "r", stdin);
	freopen ("atom.out", "w", stdout);
	//freopen ("atom.log.out", "w", stderr);
	
	int N, M;
	scanf ("%d%d", &N, &M);
	
	for (int i = 1; i <= N; i ++) {
		Z m;
		scanf ("%lld", &m);
		root = merge (root, alloc (&pool, m));
		
		//dump (root);
		//fprintf (stderr, "------\n");
	}
	
	while (M --) {
		char cmd[20];
		scanf ("%s", cmd);
		switch (cmd[1]) {
			case 'n': { // insert
				int pos; Z val;
				scanf ("%d%lld", &pos, &val);
				T *a, *b;
				split (root, pos, &a, &b);
				root = merge (merge (a, alloc (&pool, val)), b);
			}; break;
			case 'e': { // merge
				int pos; Z val;
				scanf ("%d%lld", &pos, &val);
				T *a, *b, *c;
				split (root, pos - 1, &a, &b);
				split (b, 2, &b, &c);
				root = merge (merge (a, alloc (&pool, val)), c);
			}; break;
			case 'i': { // min
				int l, r;
				scanf ("%d%d", &l, &r);
				T *a, *b, *c;
				split (root, l - 1, &a, &b);
				split (b, r - l + 1, &b, &c);
				printf ("%lld\n", b->r.mindiff);
				root = merge ( merge (a, b), c );
			}; break;
			case 'a': { // max
				int l, r;
				scanf ("%d%d", &l, &r);
				T *a, *b, *c;
				split (root, l - 1, &a, &b);
				split (b, r - l + 1, &b, &c);
				printf ("%lld\n", b->r.mx - b->r.mn);
				root = merge ( merge (a, b), c );
			}; break;
		}
		//dump (root);
		//fprintf (stderr, "------\n");
	}
	return 0;
}
