#include <iostream>
#include <cstdio>
#include <cmath>

#define MAXN 1000010

using namespace std;

int n, m;

struct node {
	int val;
	int size;
	node *father;
	node *ch[2];
	int minV;
	int maxV;
	int flag;
} pool[MAXN], *top[2], *root[2];
int cnt = 0;

int d[MAXN];

int size(node *now) {
	if(now) return now -> size;
	return 0;
}

void update(node *now) {
	now -> size = 1;
	now -> minV = abs(now -> val);
	now -> maxV = abs(now -> val);
	if(now -> ch[0] && !now -> ch[0] -> flag) {
		now -> size += size(now -> ch[0]);
		now -> minV = min(now -> minV, abs(now -> ch[0] -> minV));
		now -> maxV = max(now -> maxV, abs(now -> ch[0] -> maxV));
	}
	if(now -> ch[1] && !now -> ch[1] -> flag) {
		now -> size += size(now -> ch[1]);
		now -> minV = min(now -> minV, abs(now -> ch[1] -> minV));
		now -> maxV = max(now -> maxV, abs(now -> ch[1] -> maxV));
	}
}

void rotate(node *now, int type, int tag) { //type = 0RIGHT 1LEFT
	node *f = now -> father;
	node *gf = now -> father -> father;
	node *c = now -> ch[!type];
	if(root[tag] == f) root[tag] = now;
	if(c) c -> father = f;
	now -> father = gf;
	now -> ch[!type] = f;
	f -> father = now;
	f -> ch[type] = c;
	gf -> ch[f == gf -> ch[1]] = now;
	update(f);
	update(now);
}

void splay(node *now, node *tar, int tag) {
	while(now -> father != tar) {
		if(now -> father -> father == tar)
			rotate(now, now == now -> father -> ch[1], tag);
		else {
			node *f = now -> father;
			node *gf = now -> father -> father;
			int type = now == now -> father -> ch[1];
			if(type == (f == gf -> ch[1])) rotate(f, type, tag), rotate(now, type, tag);
			else rotate(now, type, tag), rotate(now, !type, tag);
		}
	}
}

node *find(node *now, int x) {
	if(size(now -> ch[0]) >= x + 1) return find(now -> ch[0], x);
	else if(size(now -> ch[0]) <= x - 1) return find(now -> ch[1], x - 1 - size(now -> ch[0]));
	else return now;
}

void insert(int x, int e, int tag) {
	node *now = find(root[tag], x);
	splay(now, top[tag], tag);
	now = find(root[tag], x + 1);
	splay(now, root[tag], tag);
	node *cur = &pool[++cnt];
	now -> ch[0] = cur;
	cur -> val = e;
	cur -> father = now;
	update(cur);
	update(now);
	update(now -> father);
}

void del(int x, int tag) {
	node *now = find(root[tag], x - 1);
	splay(now, top[tag], tag);
	now = find(root[tag], x + 1);
	splay(now, root[tag], tag);
	now -> ch[0] = NULL;
	update(now);
	update(now -> father);
}

int query(int x, int y, int type, int tag) {
	node *now = find(root[tag], x - 1);
	splay(now, top[tag], tag);
	now = find(root[tag], y + 1);
	splay(now, root[tag], tag);
	return type ? now -> ch[0] -> maxV : now -> ch[0] -> minV;
}

void printSplay(node *now) {
	if(now -> ch[0] && !now -> ch[0] -> flag) printSplay(now -> ch[0]);
	printf("%d %d %d\n", now -> val, now -> father -> val, now -> size);
	if(now -> ch[1] && !now -> ch[1] -> flag) printSplay(now -> ch[1]);
}

int main() {
	freopen("atom.in", "r", stdin);
	freopen("atom.out", "w", stdout);
	scanf("%d %d", &n, &m);
	for(int i = 0; i <= 1; i++) {
		top[i] = &pool[++cnt];
		root[i] = &pool[++cnt];
		top[i] -> ch[0] = root[i];
		root[i] -> father = top[i];
		root[i] -> size = 1;
		root[i] -> flag = 1;
	}
	node *now;
	for(int i = 1; i <= n; i++) {
		scanf("%d", &d[i]);
		now = &pool[++cnt];
		root[0] -> ch[1] = now;
		now -> val = d[i];
		now -> size = 1;
		now -> father = root[0];
		splay(now, top[0], 0);
		now = &pool[++cnt];
		root[1] -> ch[1] = now;
		now -> val = d[i] - d[i - 1];
		now -> size = 1;
		now -> father = root[1];
		splay(now, top[1], 1);
	}
	for(int i = 0; i <= 1; i++) {
		now = &pool[++cnt];
		root[i] -> ch[1] = now;
		now -> size = 1;
		now -> father = root[i];
		now -> flag = 1;
		splay(now, top[i], i);
	}
	char str[100];
	int x, y;
	for(int i = 1; i <= m; i++) {
		scanf("%s %d %d", &str, &x, &y);
		if(str[1] == 'e') {
			now = find(root[0], x - 1);
			int dx1 = now -> val;
			now = find(root[0], x + 2);
			int dx2 = now -> val;
			del(x, 1);
			del(x + 1, 1);
			del(x + 2, 1);
			insert(x - 1, y - dx1, 1);
			insert(x, dx2 - y, 1);
			del(x, 0);
			del(x + 1, 0);
			insert(x - 1, y, 0);
		}
		else if(str[1] == 'n') {
			now = find(root[0], x - 1);
			int dx1 = now -> val;
			now = find(root[0], x);
			int dx = now -> val;
			del(x, 1);
			insert(x - 1, y - dx1, 1);
			insert(x, dx - y, 1);
			insert(x, y, 0);
		}
		else if(str[1] == 'i')
			printf("%d\n", query(x, y, 0, 1));
		else if(str[1] == 'a')
			printf("%d\n", query(x, y, 1, 0) - query(x, y, 0, 0));
	}
	return 0;
}
