#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <algorithm>

using namespace std;

const int MAXN = 200000 + 100;
const int INF = 0x3f3f3f3f;

//struct State
//{
//	int le, ri, del, mx, mn;
//	State(int _l, int _r, int _d, int _mx, int _mn) : le(_l), ri(_r), del(_d), mx(_mx), mn(_mn)
//	{}
//};

struct Node
{
	int h, s, v;
	int le, ri, del, mx, mn;
	Node *lc, *rc;
	Node() {}
	Node(int _v) : h(rand()), s(1), v(_v), le(_v), ri(_v), del(INF), mx(_v), mn(_v), lc(NULL), rc(NULL)
	{}
}
*root, pool[MAXN];
int totp = 0;

Node* newNode(int v)
{
	++totp;
	pool[totp] = Node(v);
	return &pool[totp];	
}

int getS(Node *now)
{
	return now ? now->s : 0;
}

void update(Node *now)
{
	if(!now) return ;
	if(now->lc && now->rc)
	{
		now->s = now->lc->s + now->rc->s + 1;
		now->del = min(now->lc->del, now->rc->del);
		now->del = min(now->del, abs(now->lc->ri - now->v));
		now->del = min(now->del, abs(now->rc->le - now->v));
		now->mx = max(now->v, max(now->lc->mx, now->rc->mx));
		now->mn = min(now->v, min(now->lc->mn, now->rc->mn));
		now->le = now->lc->le;
		now->ri = now->rc->ri;
	}
	else if(now->lc)
	{
		now->s = now->lc->s + 1;
		now->del = min(now->lc->del, abs(now->lc->ri - now->v));
		now->mx = max(now->v, now->lc->mx);
		now->mn = min(now->v, now->lc->mn);
		now->le = now->lc->le;
		now->ri = now->v;
	}
	else if(now->rc)
	{
		now->s = now->rc->s + 1;
		now->del = min(now->rc->del, abs(now->rc->le - now->v));
		now->mx = max(now->v, now->rc->mx);
		now->mn = min(now->v, now->rc->mn);
		now->le = now->v;
		now->ri = now->rc->ri;
	}
	else
	{
		now->s = 1;
		now->le = now->ri = now->v;
		now->del = INF, now->mx = now->mn = now->v;
	}
}

Node *merge(Node *a, Node *b)
{
	if(!a) return b;
	if(!b) return a;
	if(a->h > b->h)
	{
		a->rc = merge(a->rc, b);
		update(a);
		return a;
	}
	else
	{
		b->lc = merge(a, b->lc);
		update(b);
		return b;	
	}
}

void split(Node *now, int x, Node* &A, Node* &B)
{
//	cout << now << endl;
	if(!now)
	{
		A = B = NULL;
		return ;	
	}
	int r = getS(now->lc) + 1;
	if(x < r)
	{
		Node *BB;
		split(now->lc, x, A, BB);
		now->lc = BB;
		update(now);
		B = now;
	}
	else
	{
		Node *AA;
		split(now->rc, x - r, AA, B);
		now->rc = AA;
		update(now);
		A = now;
	}
}

void print(Node *now, string s = "", int tab = 0)
{
	if(!now)
		return ;	
	for(int i = 1; i <= tab; i++)
		cout << "\t";
	cout << s;
	cout << now->v << " : " << now->del;
	cout << endl;
	print(now->lc, "l = ", tab + 1);
	print(now->rc, "r = ", tab + 1);
}

int n, m;

int main()
{
	srand(20170119);
	ios::sync_with_stdio(false);
	freopen("atom.in", "r", stdin);
	freopen("atom.out", "w", stdout);
	
	cin >> n >> m;
	for(int i = 1; i <= n; i++)
	{
		int x;
		cin >> x;
		Node *cur = newNode(x);
		root = merge(root, cur);
//		cout << root << endl;
	}
//	cout << root->mx << ' ' << root->mn << ' ' << root->del << endl;
	while(m--)
	{
		string s;
		int x, y;
		cin >> s >> x >> y;
		if(s == "merge")
		{
			Node *A, *B, *C;
			split(root, x - 1, A, C);
			split(C, 2, B, C);
			B = newNode(y);
			root = merge(A, merge(B, C));
		}
		else if(s == "insert")
		{
			Node *A, *B, *C;
			split(root, x, A, C);
			B = newNode(y);
			root = merge(A, merge(B, C));
		}
		else if(s == "max")
		{
			Node *A, *B, *C;
			split(root, x - 1, A, C);
			split(C, y - x + 1, B, C);
			int ans = B->mx - B->mn;
			root = merge(A, merge(B, C));
			cout << ans << endl;
		}
		else if(s == "min")
		{
			Node *A, *B, *C;
			split(root, x - 1, A, C);
			split(C, y - x + 1, B, C);
			int ans = B->del;
//			print(B);
			root = merge(A, merge(B, C));
			cout << ans << endl;
		}
	}
	

	return 0;
}
/*
4 3
5 8 10 2
max 1 3
min 1 3
max 2 4 
*/

/*
6 7
1 2 3 4 5 6
insert 1 3
max 2 4
merge 1 2
max 2 4
min 2 4
insert 6 1
max 5 7
*/


