
// Copyright (C) 2000-2017  Geoffrey "jeffswt" Tang.
//
// This file is part of the BJOI-WC solutions.  This program is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 3, or (at your option)
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// Potentially passing 80% of the data.

#include <iostream>
#include <cstdlib>
#include <cstdio>

#define SWTAPI inline
#define rep(_var,_begin,_end) for(int _var=_begin;_var<=_end;_var++)
#define range(_begin,_end) rep(_,_begin,_end)
using namespace std;
typedef long long lli;
const int maxn = 300010;
const lli infinit = 0x007f7f7f7f7f7f7fll;

class SplayTree
{
public:
    int arr_i[maxn][5];
    lli arr_l[maxn][6];
    int root, ncnt;
    #define lc(_x) arr_i[_x][0] // Left child
    #define rc(_x) arr_i[_x][1] // Right child
    #define ch(_x,_y) arr_i[_x][_y] // Children
    #define par(_x) arr_i[_x][2] // Parent
    #define size(_x) arr_i[_x][3] // Subtree size
    #define val(_x) arr_l[_x][0] // Value
    #define lval(_x) arr_l[_x][1] // Leftmost value in interval
    #define rval(_x) arr_l[_x][2] // Rightmost value in interval
    #define vmx(_x) arr_l[_x][3] // Maximum value in interval
    #define vmn(_x) arr_l[_x][4] // Minimum value in interval
    #define vmd(_x) arr_l[_x][5] // Minimum distance between two adjacent values
    SWTAPI int make_node(int val)
    {
        int p = ++ncnt;
        lc(p) = rc(p) = par(p) = 0;
        size(p) = 1;
        val(p) = lval(p) = rval(p) = vmx(p) = vmn(p) = val;
        vmd(p) = infinit;
        return p;
    }
    SWTAPI void update_lazy(int p)
    {
        size(p) = size(lc(p)) + 1 + size(rc(p));
        lval(p) = lc(p) ? lval(lc(p)) : val(p);
        rval(p) = rc(p) ? rval(rc(p)) : val(p);
        // Updating values' lazy
        vmx(p) = val(p);
        vmn(p) = val(p);
        vmd(p) = infinit;
        range(0,1) if (ch(p,_)) {
            vmx(p) = max(vmx(p), vmx(ch(p,_)));
            vmn(p) = min(vmn(p), vmn(ch(p,_)));
            vmd(p) = min(vmd(p), vmd(ch(p,_)));
        }
        if (lc(p)) vmd(p) = min(vmd(p), abs(rval(lc(p)) - val(p)));
        if (rc(p)) vmd(p) = min(vmd(p), abs(lval(rc(p)) - val(p)));
        return ;
    }
    SWTAPI void rotate(int p)
    {
        int q = par(p), g = par(q);
        int x = p == rc(q), y = q == rc(g);
        ch(q,x) = ch(p,!x); if (ch(q,x)) par(ch(q,x)) = q;
        ch(p,!x) = q; par(q) = p;
        if (g) ch(g,y) = p; par(p) = g;
        update_lazy(q);
        update_lazy(p);
        return ;
    }
    SWTAPI void splay(int p, int t)
    {
        // printf("splaying %d to %d\n", p, t);
        for (int q = 0; (q = par(p)) && q != t; rotate(p))
            if (par(q) && par(q) != t)
                rotate((p == rc(q)) == (q == rc(par(q))) ? q : p);
        if (t == 0) root = p;
        return ;
    }
    SWTAPI int find(int t)
    {
        int p = root;
        while (true) {
            if (size(lc(p)) >= t) {
                p = lc(p); continue; }
            t -= size(lc(p));
            if (t == 1) return p;
            t -= 1;
            p = rc(p); continue;
        }
        return p;
    }
    SWTAPI lli query_max(int l, int r)
    {
        int lp = find(l), rp = find(r+2);
        // printf("%d %d : %d %d\n", l, r+2, lp, rp);
        splay(lp, 0);
        splay(rp, root);
        lli res = vmx(lc(rp)) - vmn(lc(rp));
        return res;
    }
    SWTAPI lli query_min(int l, int r)
    {
        int lp = find(l), rp = find(r+2);
        // printf("%d %d : %d %d\n", l, r+2, lp, rp);
        splay(lp, 0);
        splay(rp, root);
        lli res = vmd(lc(rp));
        return res;
    }
    SWTAPI void insert(int l, lli v)
    {
        int lp = find(l+1), rp = find(l+2);
        splay(lp, 0);
        splay(rp, root);
        int p = make_node(v);
        // Connecting new node to tree and updating
        par(p) = rp;
        lc(rp) = p;
        update_lazy(rp);
        update_lazy(lp);
        splay(p, 0);
        return ;
    }
    SWTAPI void merge(int l, lli v)
    {
        int lp = find(l), rp = find(l+3);
        splay(lp, 0);
        splay(rp, root);
        int p = make_node(v);
        // Connecting new node to tree and eradicating old ones
        par(p) = rp;
        lc(rp) = p;
        update_lazy(rp);
        update_lazy(lp);
        splay(p, 0);
        return ;
    }
    SWTAPI int build_tree(int arr[], int l, int r)
    {
        if (l > r)
            return 0;
        int mid = (l+r) >> 1;
        int p = make_node(arr[mid - 1]);
        lc(p) = build_tree(arr, l, mid - 1);
        rc(p) = build_tree(arr, mid + 1, r);
        range(0,1) par(ch(p,_)) = p;
        update_lazy(p);
        return p;
    }
    SWTAPI void init(int n, int arr[])
    {
        arr[0] = arr[n+1] = 0;
        root = this->build_tree(arr, 1, n + 2);
        return ;
    }
    SWTAPI void debug(void)
    {
        printf("================ DEBUGGER ================\n");
        printf("#%d -> is root, tree size %d\n", root, size(root));
        for (int i = 1; i <= ncnt; i++)
            printf("#%d: ln %d %d %d; sz %d v %lld:%lld:%lld lz %lld:%lld:%lld\n",
                i, par(i), lc(i), rc(i), size(i), val(i), lval(i), rval(i),
                vmx(i), vmn(i), vmd(i));
        return ;
    }
} st;

int n, m;
int E[maxn];
char str[32];

int main(int argc, char** argv)
{
    freopen("atom.in", "r", stdin);
    freopen("atom.out", "w", stdout);
    scanf("%d%d", &n, &m);
    for (int i = 1; i <= n; i++)
        scanf("%d", &E[i]);
    // Constructing structural database
    st.init(n, E);
    for (int i = 1; i <= m; i++) {
        scanf("%s", str);
        int a, b;
        scanf("%d%d", &a, &b);
        if (str[0] == 'm' && str[1] == 'e') {
            st.merge(a, b);
        } else if (str[0] == 'i') {
            st.insert(a, b);
        } else if (str[0] == 'm' && str[1] == 'a') {
            lli res = st.query_max(a, b);
            printf("%lld\n", res);
        } else if (str[0] == 'm' && str[1] == 'i') {
            lli res = st.query_min(a, b);
            printf("%lld\n", res);
        }
        continue;
    }
    return 0;
}

