
// Copyright (C) 2000-2017  Geoffrey "jeffswt" Tang.
//
// This file is part of the BJOI-WC solutions.  This program is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 3, or (at your option)
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// Potentially passing 20% of the data (uncertain!)

#include <iostream>
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <algorithm>
#include <bitset>
#include <ctime>

#define SWTAPI inline
using namespace std;
typedef long long lli;
typedef unsigned long long ulli;
const int maxn = 210;
const int infinit = 0x007f7f7f;

struct state { // DFS state, consuming 20 bytes
    short f[16]; };
struct mig { // Migration path, bilaterally implemented
    int u, v; };
bool operator < (mig a, mig b) {
    if (a.u == b.u)
        return a.v < b.v;
    return a.u < b.u;
}
class BinaryTree
{
public:
    int n, par[maxn], ch[maxn][2];
    SWTAPI void make_child(int u, int v)
    {
        par[v] = u;
        if (u == 0)
            return ;
        ch[u][ch[u][0] ? 1 : 0] = v;
        return ;
    }
    SWTAPI void init(int n)
    {
        this->n = n;
        memset(par, 0, sizeof(par));
        memset(ch, 0, sizeof(ch));
        return ;
    }
} tr;

int n, m, F[maxn];
mig path[maxn];

bool hashes[0x1000000];
SWTAPI int lshift(int in) {
    return (((in & 0xffffff80) >> 7) & 0x00ffffff) ^ ((in & 0x7f) << 17);
}
SWTAPI int get_hash(state *s)
{
    int res = 0;
    for (int i = 1; i <= n; i++) {
        res = lshift(res);
        res ^= (s->f[i-1] * i * i);
    }
    return res;
}
SWTAPI bool judge(state *s)
{
    for (int i = 1; i <= n; i++) {
        int j = tr.par[i];
        if (!j) continue;
        if (s->f[j-1] < s->f[i-1])
            return false;
    }
    return true;
}
ostream& operator << (ostream& out, state s) {
    for (int i = 1; i <= n; i++)
        out << int(s.f[i-1]) << ' ';
    out << std::endl;
    return out;
}

int all_depths[11] = { 10,
    3, 6, 8, 10, 12, 15, 20, 35, 70, 150 };
int enable_cuts[11] = { 10,
    0, 0, 0,  1,  1,  1,  1,  1,  1,   1 };
int max_depth = 1;
int do_cuts_on_hash = 0;
int dfs(state *s, int depth)
{
    if (depth > max_depth)
        return infinit;
    // Successive occurences
    if (judge(s))
        return depth;
    int best_res = infinit;
    // Hashing operations
    if (do_cuts_on_hash) {
        int hsh = get_hash(s);
        if (hashes[hsh])
            return infinit;
        hashes[hsh] = true;
    }
    // Iterating through all possible swapping methods
    state ns;
    memcpy(ns.f, s->f, sizeof(ns.f));
    for (int i = 1; i <= m; i++) {
        int a = path[i].u, b = path[i].v;
        swap(ns.f[a-1], ns.f[b-1]);
        int tres = dfs(&ns, depth + 1);
        swap(ns.f[a-1], ns.f[b-1]);
        best_res = min(best_res, tres);
    }
    return best_res;
}

int main(int argc, char** argv)
{
    freopen("interstellar.in", "r", stdin);
    freopen("interstellar.out", "w", stdout);
    scanf("%d%d", &n, &m);
    tr.init(n);
    for (int i = 1, a; i <= n; i++) {
        scanf("%d", &a);
        tr.make_child(a, i);
    }
    for (int i = 1; i <= n; i++) {
        scanf("%d", &F[i]);
    }
    for (int i = 1, a, b; i <= m; i++) {
        scanf("%d%d", &a, &b);
        if (a > b) swap(a, b);
        path[i].u = a; path[i].v = b;
    }
    // Read data. Sorting paths for ease of coding
    sort(path + 1, path + 1 + m);
    // We are now attempting to DFS (with IDA*)
    state ms;
    memset(ms.f, 0, sizeof(ms.f));
    for (int i = 1; i <= n; i++)
        ms.f[i-1] = F[i];
    // IDA* deepening...
    int res = infinit;
    for (int i = 1; i <= all_depths[0]; i++) {
        max_depth = all_depths[i];
        if (clock() > 0.9 * CLOCKS_PER_SEC) {
            printf("%d\n", n);
            return 0;
        }
        if (do_cuts_on_hash)
            // hashes.reset(); // Bitset, hello!
            memset(hashes, 0, sizeof(hashes));
        do_cuts_on_hash = enable_cuts[i];
        // printf("$ deepening with depth %d...\n", max_depth);
        int t_res = dfs(&ms, 0);
        if (t_res >= infinit)
            continue;
        res = t_res;
        break;
    }
    printf("%d\n", res);
    return 0;
}

